import tomlkit


def fill_tool_section(previous_content, tool_name, section_text):
    """TOML Helper ensuring that a [tool.<tool_name>] section contains <section_text>"""
    current_state = tomlkit.parse(previous_content)

    tool_table = tomlkit.parse(section_text)["tool"][tool_name]

    if "tool" not in current_state:
        tool_table = tomlkit.table()
        # This is not public API but prevents a useless "[tool]" line
        tool_table._is_super_table = True
        current_state.add("tool", tool_table)

    if tool_name not in current_state["tool"]:
        current_state["tool"].add(tool_name, tool_table)
    else:
        current_state["tool"][tool_name] = tool_table

    return tomlkit.dumps(current_state)



import argparse
import os
import sys

import tomlkit

from . import exceptions
from . import sync


try:
    import importlib_metadata
except ImportError:
    # Use third party library on Python < 3.8
    import importlib.metadata as importlib_metadata


def parse_args():
    parser = argparse.ArgumentParser("copyist")
    parser.add_argument(
        "--version", action="version", version=importlib_metadata.version("copyist")
    )

    parser.add_argument(
        "--config",
        "-c",
        default="pyproject.toml",
        help="Configuration file (defaults to %(default)s)",
    )
    parser.add_argument(
        "--verbose",
        "-v",
        action="store_true",
        default=False,
        help="Show the diff produced at each stage",
    )
    parser.add_argument(
        "--dry-run", action="store_true", default=False, help="Do not overwrite files"
    )
    return parser.parse_args()


def read_configuration(filename):
    if not os.path.exists(filename):
        raise exceptions.ConfigurationException(
            f"Could not find configuration file {filename}"
        )

    with open(filename) as conf_file:
        conf = tomlkit.parse(conf_file.read())

    if not conf.get("tool") or not conf["tool"].get("copyist"):
        raise exceptions.ConfigurationException(
            f"Section [tool.copyist] missing from {filename}"
        )

    copyist_conf = conf["tool"]["copyist"]
    return copyist_conf.get("files", {}), copyist_conf.get("context", {})


def main():
    options = parse_args()
    try:
        file_generators, context = read_configuration(options.config)
    except exceptions.ConfigurationException as e:
        print(e.args[0])
        sys.exit(1)

    try:
        sync.sync_files(
            file_generators, context, verbose=options.verbose, dry_run=options.dry_run,
        )
    except exceptions.SyncException as e:
        print(e.args[0])
        sys.exit(1)


if __name__ == "__main__":
    main()



import difflib
import importlib
import os.path

from . import exceptions


def _import_name(generator):
    """Import a class from its module name."""
    # A generator 'path'
    try:
        mod_name, func_name = generator.rsplit(".", 1)
    except ValueError:
        raise ValueError("Invalid generator class name %s" % generator)

    try:
        mod = importlib.import_module(mod_name)
    except ImportError as e:
        raise ValueError("Error importing generator module %s: '%s'" % (mod_name, e))

    try:
        return getattr(mod, func_name)
    except AttributeError:
        raise ValueError(
            "Module '%s' does not define a '%s' class" % (mod_name, func_name)
        )


def sync_files(file_generators, context, *, verbose=False, dry_run=False):
    for filename, generators in file_generators.items():
        if os.path.exists(filename):
            with open(filename) as f:
                file_content = f.read()
        else:
            file_content = ""

        for generator_name in generators:
            try:
                generator = _import_name(generator_name)
            except ValueError as e:
                raise exceptions.SyncException(
                    f"Could not find generator {generator_name} for {filename}: {e.args[0]}"
                )
            new_file_content = generator(file_content, context=context)
            if new_file_content != file_content:
                print(f"{filename}: updated by {generator_name}")
                if verbose:
                    diff = "\n".join(
                        difflib.unified_diff(
                            file_content.splitlines(), new_file_content.splitlines()
                        )
                    )
                    print(diff)
                file_content = new_file_content
            else:
                print(f"{filename}: up-to-date for {generator_name}")

        if dry_run:
            print(f"{filename}: skipping write due to --dry-run")
            continue

        with open(filename, "w") as f:
            f.write(new_file_content)
